package org.arefgard.icerya.container.injection.impl;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.arefgard.icerya.aop.AspectProxyFactory;
import org.arefgard.icerya.container.BeanCreateException;
import org.arefgard.icerya.container.ContainerException;
import org.arefgard.icerya.container.IceryaContainer;
import org.arefgard.icerya.container.def.AspectDef;
import org.arefgard.icerya.container.def.BeanDef;
import org.arefgard.icerya.container.def.BeanResourceDef;
import org.arefgard.icerya.container.def.ElementDef;
import org.arefgard.icerya.container.def.EntryDef;
import org.arefgard.icerya.container.def.PropertyDef;
import org.arefgard.icerya.container.def.ReferenceResourceDef;
import org.arefgard.icerya.container.def.ResourceDef;
import org.arefgard.icerya.container.def.ValueResourceDef;
import org.arefgard.icerya.container.injection.InjectionStrategy;
import org.arefgard.icerya.container.util.ReflectionUtil;

/**
 * SettergCWFNV헪B
 * 
 * @author Takashi Yamashina
 *
 */
public class SetterInjectionStrategy extends InjectionStrategy {

	/** K[ */
	private static final Log log = LogFactory.getLog(SetterInjectionStrategy.class);
	
	@Override
	protected void generate(BeanDef beanDef) throws ContainerException {
		if(this.injectObject == null) {
			// IuWFNg݂Ȃꍇ
			if(beanDef == null) {
				log.error("Illegal BeanDef.");
				throw new ContainerException("Illegal Argument.");
			}
			String id = beanDef.getBeanId();
			Class cls = null;
			
			try {
				cls = Class.forName(beanDef.getBeanClass());
			}catch(Exception e) {
				throw new ContainerException("class[" + beanDef.getBeanClass() + "] is not found");
			}
			
			// AOP̎s
			List<AspectDef> aspects = beanDef.getAspects();
			if(aspects != null && aspects.size() != 0) {
				AspectProxyFactory factory = new AspectProxyFactory(cls);
				for(AspectDef aspectDef : aspects) {
					List<String> patterns = new java.util.ArrayList<String>();
					patterns.add(aspectDef.getPointcut());
					factory.addInterceptor(patterns, beanDef.getContainer().getBean(aspectDef.getAdvice()));
				}
				
				cls = factory.getProxy();
			}
			
			// IuWFNg̎ނ
			if(this.isListObject(cls)) {
				// Xg
				beanDef.setListObject(true);
			}else if(this.isMapObject(cls)) {
				// }bv
				beanDef.setMapObject(true);
			}
			
			if(log.isDebugEnabled()) {
				log.debug("  id = " + id + ", className = " + cls.getName());
			}
			
			String context = beanDef.getContext();
			try {
				
				this.injectObject = ReflectionUtil.generate(cls);
				if(context != null) {
					this.injectObject = ReflectionUtil.convertObject(this.injectObject, context);
				}
			} catch (Exception e) {
				log.error("Bean creation failed.", e);
				throw new BeanCreateException("Bean creation failed.", e);
			}
		}
	}
	
	@Override
	protected void injection(BeanDef beanDef) throws ContainerException {
		
		IceryaContainer container = beanDef.getContainer();
		if(beanDef.isListObject()) {
			// ListIuWFNg̏ꍇ
			List<ElementDef> elements = beanDef.getElements();
			List list = (List)this.injectObject;
			try {
				for(ElementDef element : elements) {
					Object value = null;
					ResourceDef resourceDef = element.getResourceDef();
					if(resourceDef instanceof ValueResourceDef) {
						ValueResourceDef valueDef = (ValueResourceDef)resourceDef;
						value = valueDef.getValue();
					}else if(resourceDef instanceof ReferenceResourceDef) {
						ReferenceResourceDef refDef = (ReferenceResourceDef)resourceDef;
						value = container.getBean(refDef.getRef());
					}else if(resourceDef instanceof BeanResourceDef) {
						BeanResourceDef beanResourceDef = (BeanResourceDef)resourceDef;
						InjectionStrategy injection = beanResourceDef.getInjectionStrategy();
						injection.inject(beanResourceDef);
						value = injection.getInjectObject();
					}
					list.add(value);
				}
			}catch(Exception e) {
				log.error("Bean creation failed.", e);
				throw new BeanCreateException("Bean creation failed.", e);
			}
			this.injectObject = list;
		}else if(beanDef.isMapObject()) {
			// MapIuWFNg̏ꍇ
			List<EntryDef> entries = beanDef.getEntries();
			Map map = (Map)this.injectObject;
			try {
				for(EntryDef entry : entries) {
					String key = entry.getKey();
					Object value = null;
					ResourceDef resourceDef = entry.getResourceDef();
					if(resourceDef instanceof ValueResourceDef) {
						ValueResourceDef valueDef = (ValueResourceDef)resourceDef;
						value = valueDef.getValue();
					}else if(resourceDef instanceof ReferenceResourceDef) {
						ReferenceResourceDef refDef = (ReferenceResourceDef)resourceDef;
						value = container.getBean(refDef.getRef());
					}else if(resourceDef instanceof BeanResourceDef) {
						BeanResourceDef beanResourceDef = (BeanResourceDef)resourceDef;
						InjectionStrategy injection = beanResourceDef.getInjectionStrategy();
						injection.inject(beanResourceDef);
						value = injection.getInjectObject();
					}
					map.put(key, value);
				}
			}catch(Exception e) {
				log.error("Bean creation failed.", e);
				throw new BeanCreateException("Bean creation failed.", e);
			}
			this.injectObject = map;
		}else {
			// ʏ̃IuWFNg
			Map<String, PropertyDef> prop = beanDef.getProperties();
			try {
				
				// vpeB̏
				if (prop.size() > 0) {
					// property
					for (Iterator<String> itr = prop.keySet().iterator(); itr.hasNext();) {
						String key = itr.next();
						Object value = null;
						
						PropertyDef propertyDef = prop.get(key);
						
						ResourceDef resourceDef = propertyDef.getResourceDef();
						
						if(resourceDef instanceof ValueResourceDef) {
							ValueResourceDef valueDef = (ValueResourceDef)resourceDef;
							value = valueDef.getValue();
						}else if(resourceDef instanceof ReferenceResourceDef) {
							ReferenceResourceDef refDef = (ReferenceResourceDef)resourceDef;
							value = container.getBean(refDef.getRef());
						}else if(resourceDef instanceof BeanResourceDef) {
							BeanResourceDef beanResourceDef = (BeanResourceDef)resourceDef;
							InjectionStrategy injection = beanResourceDef.getInjectionStrategy();
							injection.inject(beanResourceDef);
							value = injection.getInjectObject();
						}
						ReflectionUtil.setProperty(this.injectObject, key, value);
					}
				}
			} catch (Exception e) {
				log.error("Bean creation failed.", e);
				throw new BeanCreateException("Bean creation failed.", e);
			}
			
			// init-methods
			try {
				String initMethod = beanDef.getInitMethod();
				if(initMethod != null && !initMethod.equals("")) {
					// destroyMethods
					ReflectionUtil.execute(this.injectObject, initMethod, null);
				}
			}catch(Exception e) {
				e.printStackTrace();
				throw new ContainerException("", e);
			}
		}
	}
	
	/**
	 * Ώۂ̃NXListC^tF[XĂ邩ǂ
	 * ׂB
	 * 
	 * @param cls
	 * @return
	 */
	private boolean isListObject(Class cls) {
		Class[] ifcs = cls.getInterfaces();
		
		for(Class ifc : ifcs) {
			if(ifc.equals(List.class)) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Ώۂ̃NXMapC^tF[XĂ邩ǂ
	 * ׂB
	 * 
	 * @param cls
	 * @return
	 */
	private boolean isMapObject(Class cls) {
		Class[] ifcs = cls.getInterfaces();
		
		for(Class ifc : ifcs) {
			if(ifc.equals(Map.class)) {
				return true;
			}
		}
		return false;
	}

}
